==> xmlroff-0.3.3.tar.gz: xmlroff-0.3.3/property/fo-property-source-document.c <==
/* Fo
 * fo-property-source-document.c: 'source-document' property
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-source-document.c,v 1.3 2003/05/16 10:40:08 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-context.h"
#include "datatype/fo-datatype.h"
#include "property/fo-property-private.h"
#include "property/fo-property-font-size.h"
#include "property/fo-property-source-document.h"

/* Inherited: FALSE */
/* Shorthand: FALSE */
/* source-document */
/* <uri-specification> [<uri-specification>]* | none | inherit */
GType
fo_enum_source_document_get_type (void)
{
  static GType etype = 0;
  if (etype == 0)
    {
      static const GEnumValue values[] = {
        { FO_ENUM_SOURCE_DOCUMENT_NONE, "FO_ENUM_SOURCE_DOCUMENT_NONE", "none" },
        { 0, NULL, NULL }
      };
      etype = g_enum_register_static ("FoEnumSourceDocument", values);
    }
  return etype;
}

struct _FoPropertySourceDocument
{
  FoProperty parent_instance;
};

struct _FoPropertySourceDocumentClass
{
  FoPropertyClass parent_class;
};

static void fo_property_source_document_init         (FoPropertySourceDocument      *property_source_document);
static void fo_property_source_document_base_init   (FoPropertySourceDocumentClass *klass);
static void fo_property_source_document_base_finalize   (FoPropertySourceDocumentClass *klass);
static void fo_property_source_document_class_init   (FoPropertySourceDocumentClass *klass);
static void fo_property_source_document_finalize     (GObject       *object);

static FoDatatype* fo_property_source_document_resolve_enum (const gchar *token,
                                                             FoContext   *context,
                                                             GError     **error);
static FoDatatype* fo_property_source_document_validate (FoDatatype *datatype,
                                                         FoContext  *context,
                                                         GError    **error);

static const gchar class_name[] = "source-document";
static gpointer parent_class;
static FoDatatype *enum_none;

/**
 * fo_property_source_document_get_type:
 * 
 * Register the #FoPropertySourceDocument type if not already registered and
 * return its #GType value.
 * 
 * Return value: #GType of #FoPropertySourceDocument.
 **/
GType
fo_property_source_document_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertySourceDocumentClass),
        (GBaseInitFunc) fo_property_source_document_base_init,
        (GBaseFinalizeFunc) fo_property_source_document_base_finalize,
        (GClassInitFunc) fo_property_source_document_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertySourceDocument),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_source_document_init,
	NULL		/* value_table */
      };

      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            class_name,
                                            &object_info, 0);
    }

  return object_type;
}

/**
 * fo_property_source_document_init:
 * @source_document: #FoPropertySourceDocument object to initialise.
 * 
 * Implements #GInstanceInitFunc for #FoPropertySourceDocument.
 **/
void
fo_property_source_document_init (FoPropertySourceDocument *source_document)
{
  FO_PROPERTY (source_document)->value =
    g_object_ref (enum_none);
}

/**
 * fo_property_source_document_base_init:
 * @klass: #FoPropertySourceDocumentClass object to initialise.
 * 
 * Implements #GBaseInitFunc for #FoPropertySourceDocumentClass.
 **/
void
fo_property_source_document_base_init (FoPropertySourceDocumentClass *klass)
{
  enum_none =
    g_object_new (FO_TYPE_ENUM,
                  "enum-class",
                  g_type_class_ref (FO_TYPE_ENUM_SOURCE_DOCUMENT),
                  "value",
                  FO_ENUM_SOURCE_DOCUMENT_NONE,
                  NULL);
}

/**
 * fo_property_source_document_base_finalize:
 * @klass: #FoPropertySourceDocumentClass object to finalise.
 * 
 * Implements #GBaseFinalizeFunc for #FoPropertySourceDocumentClass.
 **/
void
fo_property_source_document_base_finalize (FoPropertySourceDocumentClass *klass)
{
  g_object_unref (enum_none);
}

/**
 * fo_property_source_document_class_init:
 * @klass: #FoPropertySourceDocumentClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoPropertySourceDocumentClass.
 **/
void
fo_property_source_document_class_init (FoPropertySourceDocumentClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoPropertyClass *property_class = FO_PROPERTY_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = fo_property_source_document_finalize;

  property_class->is_inherited = FALSE;
  property_class->is_shorthand = FALSE;
  property_class->resolve_enum =
    fo_property_source_document_resolve_enum;
  property_class->validate =
    fo_property_source_document_validate;
  property_class->get_initial =
    fo_property_source_document_get_initial;
}

/**
 * fo_property_source_document_finalize:
 * @object: #FoPropertySourceDocument object to finalize.
 * 
 * Implements #GObjectFinalizeFunc for #FoPropertySourceDocument.
 **/
void
fo_property_source_document_finalize (GObject *object)
{
  FoPropertySourceDocument *source_document;

  source_document = FO_PROPERTY_SOURCE_DOCUMENT (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_property_source_document_new:
 * 
 * Creates a new #FoPropertySourceDocument initialized to default value.
 * 
 * Return value: the new #FoPropertySourceDocument.
 **/
FoProperty*
fo_property_source_document_new (void)
{
  FoProperty* source_document;

  source_document =
    FO_PROPERTY (g_object_new (fo_property_source_document_get_type (),
                               NULL));

  return source_document;
}

/**
 * fo_property_source_document_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_source_document_resolve_enum (const gchar *token,
                                          FoContext   *context,
                                          GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "none") == 0)
    {
      return g_object_ref (enum_none);
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   class_name,
		   token);
      return NULL;
    }
}

/**
 * fo_property_source_document_validate:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property.
 * @context:  #FoContext object from which to possibly inherit values.
 * @error:    Information about any error that has occurred.
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed.
 * 
 * Return value: Valid datatype value or NULL.
 **/
FoDatatype*
fo_property_source_document_validate (FoDatatype *datatype,
                                      FoContext  *context,
                                      GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_URI_SPECIFICATION (datatype))
    {
      return datatype;
    }
  else if (FO_IS_URI_SPECIFICATION (datatype))
    {
      return datatype;
    }
  else if (FO_IS_ENUM (datatype))
    {
      if ((G_ENUM_CLASS_TYPE (fo_enum_get_enum_class (datatype))
           == FO_TYPE_ENUM_SOURCE_DOCUMENT) &&
	  (g_enum_get_value (fo_enum_get_enum_class (datatype),
			     fo_enum_get_value (datatype)) != NULL))
	{
	  return datatype;
	}
      else
	{
	  gchar *datatype_sprintf = fo_object_sprintf (datatype);

	  g_set_error (error,
		       FO_FO_ERROR,
		       FO_FO_ERROR_DATATYPE,
		       _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		       class_name,
		       datatype_sprintf,
		       g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

	  g_object_unref (datatype);

	  g_free (datatype_sprintf);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
        fo_property_source_document_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
        fo_property_source_document_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else
    {
      gchar *datatype_sprintf = fo_object_sprintf (datatype);

      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   class_name,
		   datatype_sprintf,
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      g_free (datatype_sprintf);

      return NULL;
    }
}

/**
 * fo_property_source_document_get_initial:
 * 
 * Get an instance of the property with the correct initial value.
 * 
 * Return value: An instance of the property.
 **/
FoProperty*
fo_property_source_document_get_initial (void)
{
  static FoProperty *source_document = NULL;

  if (source_document == NULL)
    {
      source_document = fo_property_source_document_new ();
    }

  return source_document;
}

==> xmlroff-0.5.0.tar.gz: xmlroff-0.5.0/property/fo-property-source-document.c <==
/* Fo
 * fo-property-source-document.c: 'source-document' property
 *
 * Copyright (C) 2001-2005 Sun Microsystems
 *
 * $Id: fo-property-source-document.c,v 1.4 2005/12/06 11:06:16 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "libfo/fo-utils.h"
#include "fo-context.h"
#include "datatype/fo-datatype.h"
#include "property/fo-property-private.h"
#include "property/fo-property-font-size.h"
#include "property/fo-property-source-document.h"

/* source-document */
/* Inherited: FALSE */
/* Shorthand: FALSE */
/* <uri-specification> [<uri-specification>]* | none | inherit */
/* Initial value: none */

struct _FoPropertySourceDocument
{
  FoProperty parent_instance;
};

struct _FoPropertySourceDocumentClass
{
  FoPropertyClass parent_class;
};

static void fo_property_source_document_init         (FoPropertySourceDocument      *property_source_document);
static void fo_property_source_document_class_init   (FoPropertySourceDocumentClass *klass);
static void fo_property_source_document_finalize     (GObject       *object);

static FoDatatype * fo_property_source_document_resolve_enum (const gchar *token,
                                                              FoContext   *context,
                                                              GError     **error);
static FoDatatype * fo_property_source_document_validate (FoDatatype *datatype,
                                                          FoContext  *context,
                                                          GError    **error);

static const gchar class_name[] = "source-document";
static gpointer parent_class;

/**
 * fo_property_source_document_get_type:
 * 
 * Register the #FoPropertySourceDocument type if not already registered and
 * return its #GType value.
 * 
 * Return value: #GType of #FoPropertySourceDocument.
 **/
GType
fo_property_source_document_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoPropertySourceDocumentClass),
        NULL,           /* base_init */
        NULL,           /* base_finalize */
        (GClassInitFunc) fo_property_source_document_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoPropertySourceDocument),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_property_source_document_init,
	NULL		/* value_table */
      };

      object_type = g_type_register_static (FO_TYPE_PROPERTY,
                                            class_name,
                                            &object_info, 0);
    }

  return object_type;
}

/**
 * fo_property_source_document_init:
 * @source_document: #FoPropertySourceDocument object to initialise.
 * 
 * Implements #GInstanceInitFunc for #FoPropertySourceDocument.
 **/
void
fo_property_source_document_init (FoPropertySourceDocument *source_document)
{
  FO_PROPERTY (source_document)->value =
    g_object_ref (fo_enum_get_enum_by_nick ("none"));
}

/**
 * fo_property_source_document_class_init:
 * @klass: #FoPropertySourceDocumentClass object to initialise.
 * 
 * Implements #GClassInitFunc for #FoPropertySourceDocumentClass.
 **/
void
fo_property_source_document_class_init (FoPropertySourceDocumentClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  FoPropertyClass *property_class = FO_PROPERTY_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = fo_property_source_document_finalize;

  property_class->is_inherited = FALSE;
  property_class->is_shorthand = FALSE;
  property_class->resolve_enum =
    fo_property_source_document_resolve_enum;
  property_class->validate =
    fo_property_source_document_validate;
  property_class->get_initial =
    fo_property_source_document_get_initial;
}

/**
 * fo_property_source_document_finalize:
 * @object: #FoPropertySourceDocument object to finalize.
 * 
 * Implements #GObjectFinalizeFunc for #FoPropertySourceDocument.
 **/
void
fo_property_source_document_finalize (GObject *object)
{
  FoPropertySourceDocument *source_document;

  source_document = FO_PROPERTY_SOURCE_DOCUMENT (object);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


/**
 * fo_property_source_document_new:
 * 
 * Creates a new #FoPropertySourceDocument initialized to default value.
 * 
 * Return value: the new #FoPropertySourceDocument.
 **/
FoProperty*
fo_property_source_document_new (void)
{
  FoProperty* source_document;

  source_document =
    FO_PROPERTY (g_object_new (fo_property_source_document_get_type (),
                               NULL));

  return source_document;
}

/**
 * fo_property_source_document_resolve_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_source_document_resolve_enum (const gchar *token,
                                          FoContext   *context,
                                          GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "none") == 0)
    {
      return g_object_ref (fo_enum_get_enum_by_nick (token));
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   class_name,
		   token);
      return NULL;
    }
}

/**
 * fo_property_source_document_validate:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property.
 * @context:  #FoContext object from which to possibly inherit values.
 * @error:    Information about any error that has occurred.
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed.
 * 
 * Return value: Valid datatype value or NULL.
 **/
FoDatatype*
fo_property_source_document_validate (FoDatatype *datatype,
                                      FoContext  *context,
                                      GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_URI_SPECIFICATION (datatype))
    {
      return datatype;
    }
  else if (FO_IS_URI_SPECIFICATION (datatype))
    {
      return datatype;
    }
  else if (FO_IS_ENUM (datatype))
    {
      FoEnumEnum value = fo_enum_get_value (datatype);

      if (value == FO_ENUM_ENUM_NONE)
	{
	  return datatype;
	}
      else
	{
	  gchar *datatype_sprintf = fo_object_sprintf (datatype);

	  g_set_error (error,
		       FO_FO_ERROR,
		       FO_FO_ERROR_ENUMERATION_TOKEN,
		       _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		       class_name,
		       datatype_sprintf,
		       g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

	  g_object_unref (datatype);

	  g_free (datatype_sprintf);

	  return NULL;
	}
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
        fo_property_source_document_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      fo_propagate_and_return_val_if_error (error, tmp_error, NULL);

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
        fo_property_source_document_resolve_enum (token, context, &tmp_error);

      g_object_unref (datatype);

      fo_propagate_and_return_val_if_error (error, tmp_error, NULL);

      return new_datatype;
    }
  else
    {
      gchar *datatype_sprintf = fo_object_sprintf (datatype);

      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   class_name,
		   datatype_sprintf,
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      g_free (datatype_sprintf);

      return NULL;
    }
}

/**
 * fo_property_source_document_get_initial:
 * 
 * Get an instance of the property with the correct initial value.
 * 
 * Return value: An instance of the property.
 **/
FoProperty*
fo_property_source_document_get_initial (void)
{
  static FoProperty *source_document = NULL;

  if (source_document == NULL)
    {
      source_document =
	fo_property_source_document_new ();
    }

  return source_document;
}
